import cv2
import os
import re

from .contours import Contour
from .frames import Frame
from .windows import Window
from .cameras import Camera
from .background_subtractors import BackgroundSubtractor
from .video_writers import VideoWriter
from .face_extractors import FaceExtractor, HAARS_DIRECTORY


def get_camera(id_or_filename):
    return Camera(id_or_filename)


def get_background_subtractor(subtractor):
    return BackgroundSubtractor(subtractor)


def get_face_extractor(extractor_classifier):
    return FaceExtractor(extractor_classifier)


def get_video_writer(camera, filename):
    size = (int(camera.frame_width), int(camera.frame_height))
    fourcc = cv2.VideoWriter_fourcc(*'XVID')
    return VideoWriter(cv2.VideoWriter(filename, fourcc, camera.fps, size))


def get_window(name):
    return Window(name)


def get_supported_subtractors():
    supported_subtractors = {}
    if hasattr(cv2, "createBackgroundSubtractorGMG"):
        supported_subtractors["GMG"] = cv2.createBackgroundSubtractorGMG
    if hasattr(cv2, "createBackgroundSubtractorKNN"):
        supported_subtractors["KNN"] = cv2.createBackgroundSubtractorKNN
    if hasattr(cv2, "createBackgroundSubtractorMOG"):
        supported_subtractors["MOG"] = cv2.createBackgroundSubtractorMOG
    if hasattr(cv2, "createBackgroundSubtractorMOG2"):
        supported_subtractors["MOG2"] = cv2.createBackgroundSubtractorMOG2

    return supported_subtractors


def get_stock_classifiers():
    for haar in os.listdir(HAARS_DIRECTORY):
        result = re.match("haarcascade_(\w+).xml", haar)
        if result:
            yield result.group(1)



import cv2

from .frames import Frame


class Camera:
    def __init__(self, id_or_filename):
        try:
            self._camera = cv2.VideoCapture(int(id_or_filename))
        except ValueError:
            self._camera = cv2.VideoCapture(id_or_filename)

    @property
    def frames(self):
        return self._camera.get(cv2.CAP_PROP_FRAME_COUNT)

    @property
    def fps(self):
        return self._camera.get(cv2.CAP_PROP_FPS)

    @property
    def frame_width(self):
        return self._camera.get(cv2.CAP_PROP_FRAME_WIDTH)

    @property
    def frame_height(self):
        return self._camera.get(cv2.CAP_PROP_FRAME_HEIGHT)

    @property
    def current_pos(self):
        return self._camera.get(cv2.CAP_PROP_POS_MSEC)

    @property
    def current_frame(self):
        return self._camera.get(cv2.CAP_PROP_POS_FRAMES)

    def seek(self, frame):
        self._camera.set(cv2.CAP_PROP_POS_FRAMES, frame)

    def read(self):
        (grabbed, frame) = self._camera.read()
        if grabbed:
            frame = Frame(frame)
        return (grabbed, frame)

    def __del__(self):
        self._camera.release()



import cv2

from .frames import Frame
from .exceptions import InvalidBackgrounSubtractor


class BackgroundSubtractor:
    def __init__(self, subtractor="MOG2"):
        if subtractor == "MOG2":
            self._subtractor = cv2.createBackgroundSubtractorMOG2()
        elif subtractor == "KNN":
            self._subtractor = cv2.createBackgroundSubtractorKNN()
        else:
            raise InvalidBackgrounSubtractor(subtractor)

    def apply(self, frame):
        result_frame = self._subtractor.apply(frame.frame)

        return Frame(result_frame)



#!/usr/bin/env python

import click
import camtones

from camtones.procs.motion import MotionDetectProcess, MotionExtractProcess, MotionExtractEDLProcess, MotionCalibrate
from camtones.procs.face import FaceDetectProcess, FaceExtractProcess
from camtones.ocv import api as ocv


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo("Version {}.{}.{}".format(*camtones.__version__))
    ctx.exit()


@click.group()
@click.option("--debug", is_flag=True, default=False, help="debug enabled")
@click.option("--version", is_flag=True, is_eager=True, default=False, help="print version", callback=print_version)
@click.pass_context
def cli(ctx, debug, version):
    ctx.obj['DEBUG'] = debug


@cli.command(help="Show a video or camera device highlighting the detected motion")
@click.argument("video-or-device")
@click.option("--exclude", help="python expression to exclude results")
@click.option("--resize", type=int, help="resize the image before detect")
@click.option("--blur", type=int, help="blur the mask before detect")
@click.option("--threshold", type=click.IntRange(0, 255), help="threshold the mask before detect")
@click.option(
    "--subtractor",
    type=click.Choice(ocv.get_supported_subtractors().keys()),
    default="MOG2" if "MOG2" in ocv.get_supported_subtractors() else "KNN",
    help="select background subtractor"
)
@click.pass_context
def motion_detect(ctx, video_or_device, exclude, resize, blur, threshold, subtractor):
    MotionDetectProcess(video_or_device, ctx.obj['DEBUG'], exclude, resize, blur, threshold, subtractor).run()


@cli.command(help="Generate video file from a video or camera device without the no-motion sections")
@click.argument("video-or-device")
@click.argument("output-file")
@click.option("--exclude", help="python expression to exclude results")
@click.option("--progress", is_flag=True, help="print progress")
@click.option("--resize", type=int, help="resize the image before detect")
@click.option("--blur", type=int, help="blur the mask before detect")
@click.option("--threshold", type=click.IntRange(0, 255), help="threshold the mask before detect")
@click.option("--show-time", is_flag=True, help="show the current time")
@click.option(
    "--subtractor",
    type=click.Choice(ocv.get_supported_subtractors().keys()),
    default="MOG2" if "MOG2" in ocv.get_supported_subtractors() else "KNN",
    help="select background subtractor"
)
@click.pass_context
def motion_extract(ctx, video_or_device, output_file, exclude, progress, resize, blur, threshold, show_time, subtractor):
    MotionExtractProcess(video_or_device, ctx.obj['DEBUG'], exclude, output_file, progress, resize, blur, threshold, show_time, subtractor).run()


@cli.command(help="Generate an EDL file from a video or camera device to skip no-motion sections")
@click.argument("video-or-device")
@click.argument("output-file")
@click.option("--exclude", help="python expression to exclude results")
@click.option("--progress", is_flag=True, help="print progress")
@click.option("--resize", type=int, help="resize the image before detect")
@click.option("--blur", type=int, help="blur the mask before detect")
@click.option("--threshold", type=click.IntRange(0, 255), help="threshold the mask before detect")
@click.option(
    "--subtractor",
    type=click.Choice(ocv.get_supported_subtractors().keys()),
    default="MOG2" if "MOG2" in ocv.get_supported_subtractors() else "KNN",
    help="select background subtractor"
)
@click.pass_context
def motion_extract_edl(ctx, video_or_device, output_file, exclude, progress, resize, blur, threshold, subtractor):
    MotionExtractEDLProcess(video_or_device, ctx.obj['DEBUG'], exclude, output_file, progress, resize, blur, threshold, subtractor).run()


@cli.command(help="Calibrate and run motion detection")
@click.argument("video")
def motion_wizard(video):
    MotionCalibrate(video).run()


@cli.command(help="Show a video or camera device highlighting the detected faces")
@click.argument("video-or-device")
@click.option("--classifier", help="path to the cascade classifier file", required=True)
@click.pass_context
def face_detect(ctx, video_or_device, classifier):
    FaceDetectProcess(video_or_device, ctx.obj['DEBUG'], classifier).run()


@cli.command(help="List the available face classifiers")
def classifiers():
    for classifier in ocv.get_stock_classifiers():
        click.echo(" - {}".format(classifier))


@cli.command(help="Generate a set of images from a video or camera device with the detected faces")
@click.argument("video-or-device")
@click.argument("output-directory")
@click.option("--classifier", help="path to the cascade classifier file", required=True)
@click.option("--progress", is_flag=True, help="print progress")
@click.pass_context
def face_extract(ctx, video_or_device, output_directory, classifier, progress):
    FaceExtractProcess(video_or_device, ctx.obj['DEBUG'], output_directory, classifier, progress).run()


if __name__ == '__main__':
    cli(prog_name="camtones", obj={})



import os
from progressbar import ProgressBar, Bar, ETA

from camtones.ocv import api as ocv


class FaceBaseProcess(object):
    def __init__(self, video, debug, classifier):
        self.debug = debug

        self.camera = ocv.get_camera(video)
        self.faceCascade = ocv.get_face_extractor(classifier)

    def run(self):
        while True:
            if not self.process_frame():
                break


class FaceDetectProcess(FaceBaseProcess):
    def __init__(self, video, debug, classifier):
        super(FaceDetectProcess, self).__init__(video, debug, classifier)
        self.window = ocv.get_window("Face detect")

    def process_frame(self):
        (grabbed, frame) = self.camera.read()

        if not grabbed:
            return False

        faces = self.faceCascade.get_faces(frame)

        for (x, y, w, h) in faces:
            frame.draw_rect((x, y), (x + w, y + h), (0, 255, 0))

        self.window.show(frame)
        return self.window.handle(self.camera)


class FaceExtractProcess(FaceBaseProcess):
    def __init__(self, video, debug, output, classifier, progress):
        super(FaceExtractProcess, self).__init__(video, debug, classifier)
        self.output = output

        self.progress = None
        if progress:
            widgets = [Bar('>'), ' ', ETA()]
            self.progress = ProgressBar(widgets=widgets, max_value=self.camera.frames)

    def process_frame(self):
        (grabbed, frame) = self.camera.read()

        if not grabbed:
            return False

        faces = self.faceCascade.get_faces(frame)

        counter = 0
        for (x, y, w, h) in faces:
            counter += 1
            croped = frame.crop_copy((x, y), (x + w, y + h))
            filename = os.path.join(self.output, "{}-{}.png".format(self.camera.current_pos, counter))
            croped.write(filename)

        if self.progress:
            self.progress.update(self.camera.current_frame)

        return True



import cv2


class Contour:
    def __init__(self, contour):
        self.contour = contour
        (self.x, self.y, self.w, self.h) = cv2.boundingRect(self.contour)
        self.area = cv2.contourArea(self.contour)

    @property
    def width(self):
        return self.w

    @property
    def height(self):
        return self.h

    @property
    def point1(self):
        return (self.x, self.y)

    @property
    def point2(self):
        return (self.x + self.w, self.y + self.h)



import cv2
import os

from .exceptions import InvalidHaarCascade

HAARS_DIRECTORY = os.path.join(os.path.dirname(os.path.dirname(__file__)), "haars")


class FaceExtractor:
    def __init__(self, classifier):
        if os.path.exists(classifier):
            self._face_extractor = cv2.CascadeClassifier(classifier)
        elif os.path.exists(os.path.join(HAARS_DIRECTORY, "haarcascade_{}.xml".format(classifier))):
            self._face_extractor = cv2.CascadeClassifier(
                os.path.join(HAARS_DIRECTORY, "haarcascade_{}.xml".format(classifier))
            )
        else:
            raise InvalidHaarCascade(classifier)

    def get_faces(self, frame):
        gray = frame.gray_copy()

        faces = self._face_extractor.detectMultiScale(
            gray.frame,
            scaleFactor=1.1,
            minNeighbors=5,
            minSize=(30, 30),
            # flags = cv2.HAAR_SCALE_IMAGE
        )
        return faces



import time
import gi
import os

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GdkPixbuf, Gdk, GLib, GObject

from progressbar import ProgressBar, Bar, ETA

from camtones.ocv import api as ocv


class MotionBaseProcess(object):
    def __init__(self, video, debug, subtractor, blur, resize, exclude, threshold):
        self.debug = debug
        self.exclude = exclude
        self.resize = resize
        self.blur = blur
        self.threshold = threshold

        self.camera = ocv.get_camera(video)
        self.fgbg = ocv.get_background_subtractor(subtractor)

        if self.debug:
            self.fgmask_window = ocv.get_window("FGMASK")

    def exclude_frame(self, contour, frame):
        data = {
            "rect": contour,
            "frame": frame,
        }

        return self.exclude and eval(self.exclude, {}, data)

    def run(self):
        while True:
            if not self.process_frame():
                break

    def is_moving(self, frame, with_contours=True):
        fgmask = self.fgbg.apply(frame)
        if self.blur:
            fgmask.blur(self.blur)

        if self.threshold:
            fgmask.threshold(self.threshold)

        if self.debug:
            self.fgmask_window.show(fgmask)

        cnts = fgmask.get_contours()

        moving = False
        included_cnts = []
        for contour in cnts:
            if self.exclude_frame(contour, frame):
                continue

            if with_contours:
                included_cnts.append(contour)
                moving = True
            else:
                return (True, [])

        return (moving, included_cnts)


class MotionDetectProcess(MotionBaseProcess):
    def __init__(self, video, debug, exclude, resize, blur, threshold, subtractor):
        super(MotionDetectProcess, self).__init__(video, debug, subtractor, blur, resize, exclude, threshold)

        self.window = ocv.get_window("Motion extract")

    def process_frame(self):
        (grabbed, frame) = self.camera.read()

        if not grabbed:
            return False

        if self.resize:
            frame = frame.resize(width=self.resize)

        (moving, cnts) = self.is_moving(frame, with_contours=True)

        if moving:
            for contour in cnts:
                frame.draw_rect(contour.point1, contour.point2, (0, 255, 0))
                frame.draw_top_label("Moving", (0, 0, 255))
        else:
            frame.draw_top_label("Not Moving", (0, 255, 0))

        self.window.show(frame)

        return self.window.handle(self.camera)


class MotionExtractProcess(MotionBaseProcess):
    def __init__(self, video, debug, exclude, output, progress, resize, blur, threshold, show_time, subtractor):
        super(MotionExtractProcess, self).__init__(video, debug, subtractor, blur, resize, exclude, threshold)
        self.output = output
        self.show_time = show_time
        self.progress = None
        if progress:
            widgets = [Bar('>'), ' ', ETA()]
            self.progress = ProgressBar(widgets=widgets, max_value=self.camera.frames)

        self.output = ocv.get_video_writer(self.camera, output)

    def process_frame(self):
        (grabbed, frame) = self.camera.read()

        if not grabbed:
            return False

        if self.resize:
            miniframe = frame.resize(width=self.resize)
        else:
            miniframe = frame

        (moving, _) = self.is_moving(miniframe, with_contours=False)

        if moving:
            if self.show_time:
                current_time = time.gmtime(int(self.camera.current_pos/1000))
                frame.draw_timer(current_time)

            self.output.write(frame)

        if self.progress:
            self.progress.update(self.camera.current_frame)

        return True


class MotionExtractEDLProcess(MotionBaseProcess):
    def __init__(self, video, debug, exclude, output, progress, resize, blur, threshold, subtractor):
        super(MotionExtractEDLProcess, self).__init__(video, debug, subtractor, blur, resize, exclude, threshold)
        self.output = output

        self.progress = None
        if progress:
            widgets = [Bar('>'), ' ', ETA()]
            self.progress = ProgressBar(widgets=widgets, max_value=self.camera.frames)

        self.output = open(output, "w")
        self.start_silence = 0

    def __del__(self):
        if self.progress:
            self.progress.finish()

    def process_frame(self):
        (grabbed, frame) = self.camera.read()

        if not grabbed:
            return False

        if self.resize:
            miniframe = frame.resize(width=self.resize)
        else:
            miniframe = frame

        (moving, _) = self.is_moving(miniframe, with_contours=False)

        if not moving and self.start_silence is None:
            self.start_silence = self.camera.current_pos/1000

        if moving and self.start_silence is not None:
            end_silence = self.camera.current_pos/1000
            self.output.write("{} {} {}\n".format(self.start_silence, end_silence, 0))
            self.start_silence = None

        if self.progress:
            self.progress.update(self.camera.current_frame)

        return True


class MotionCalibrate(object):
    def __init__(self, video):
        self.ui = UI(video)

    def run(self):
        self.ui.main()


class FrameProcessor:
    def __init__(self, camera, orig_image, mask_image):
        self.camera = camera
        self.mog2 = ocv.get_background_subtractor("MOG2")
        self.knn = ocv.get_background_subtractor("KNN")

        self.subtractor_name = "MOG2"
        self.subtractor = self.mog2

        self.blur = 0
        self.threshold = 128
        self.resize = None
        self.orig_image = orig_image
        self.mask_image = mask_image
        self.exclude = ""


    def frame_to_gtk_pixbuf(self, frame, is_gray):
        if is_gray:
            color_frame = frame.gray_to_rgb_copy()
        else:
            color_frame = frame.bgr_to_rgb_copy()

        data = color_frame.to_string()
        return GdkPixbuf.Pixbuf.new_from_data(data, GdkPixbuf.Colorspace.RGB, False, 8, frame.width, frame.height, frame.width*3, None, None)



    def extract_and_draw(self, frame, subtractor, detection, mask):
        if self.resize:
            frame = frame.resize(self.resize)

        result_mask = subtractor.apply(frame)
        if self.blur:
            result_mask.blur(self.blur)
        if self.threshold:
            result_mask.threshold(self.threshold)

        mask.set_from_pixbuf(self.frame_to_gtk_pixbuf(result_mask, True).copy())

        cnts = result_mask.get_contours()
        result_frame = frame.clone()

        included_cnts = []
        for contour in cnts:
            data = {
                "rect": contour,
                "frame": frame,
            }

            try:
                if self.exclude and eval(self.exclude, {}, data):
                    continue
            except Exception:
                pass

            result_frame.draw_rect(contour.point1, contour.point2, (0, 255, 0))

        detection.set_from_pixbuf(self.frame_to_gtk_pixbuf(result_frame, False).copy())

    def run(self):
        (grabbed, frame) = self.camera.read()
        if not grabbed:
            self.camera.seek(0)
            (_, frame) = self.camera.read()

        self.extract_and_draw(frame, self.subtractor, self.orig_image, self.mask_image)


class UI:
    def __init__(self, video):
        self.command = None
        self.builder = Gtk.Builder()
        self.builder.add_from_file(os.path.join(os.path.dirname(__file__), "gui.glade"))

        orig_image = self.builder.get_object("image_orig")
        mask_image = self.builder.get_object("image_mask")

        self.video = video
        camera = ocv.get_camera(video)
        self.frame_processor = FrameProcessor(camera, orig_image, mask_image)

        self.builder.connect_signals(self)

        window = self.builder.get_object("window")
        window.show_all()
        blur_spin = self.builder.get_object("blur_spin")
        blur_spin.set_value(0)
        threshold_spin = self.builder.get_object("threshold_spin")
        threshold_spin.set_value(128)
        resize_spin = self.builder.get_object("resize_spin")
        resize_spin.set_value(0)


    def on_quit(self, widget, event):
        Gtk.main_quit()

    def on_blur_change(self, widget, *args):
        try:
            new_blur = int(widget.get_value())
            self.frame_processor.blur = new_blur
        except ValueError:
            pass

    def on_resize_change(self, widget, *args):
        try:
            new_resize = int(widget.get_value())
            if new_resize > 0:
                self.frame_processor.resize = new_resize
            else:
                self.frame_processor.resize = None
        except ValueError:
            pass

    def on_threshold_change(self, widget, *args):
        try:
            new_threshold = int(widget.get_value())
            self.frame_processor.threshold = new_threshold
        except ValueError:
            pass

    def on_exclude_change(self, widget, *args):
        self.frame_processor.exclude = widget.get_text()

    def on_generate_motion_edl_clicked(self, widget, *args):
        self.command = MotionExtractEDLProcess(
            video=self.video,
            subtractor=self.frame_processor.subtractor_name,
            blur=self.frame_processor.blur,
            exclude=self.frame_processor.exclude,
            resize=self.frame_processor.resize,
            debug=False,
            progress=True,
            threshold=self.frame_processor.threshold,
            output="{}.edl".format(self.video)
        )
        self.builder.get_object("window").hide()
        self.builder.get_object("window").destroy()

        Gtk.main_quit()

    def on_generate_motion_video_clicked(self, widget, *args):
        self.command = MotionExtractProcess(
            video=self.video,
            subtractor=self.frame_processor.subtractor_name,
            blur=self.frame_processor.blur,
            exclude=self.frame_processor.exclude,
            resize=self.frame_processor.resize,
            debug=False,
            progress=True,
            show_time=True,
            threshold=self.frame_processor.threshold,
            output="{}-out.avi".format(self.video)
        )
        self.builder.get_object("window").hide()
        self.builder.get_object("window").destroy()

        Gtk.main_quit()

    def on_subtractor_toggle(self, widget, *args):
        if widget.get_active():
            if widget.get_label() == "MOG2":
                self.frame_processor.subtractor = self.frame_processor.mog2
                self.frame_processor.subtractor_name = "MOG2"
            elif widget.get_label() == "KNN":
                self.frame_processor.subtractor = self.frame_processor.knn
                self.frame_processor.subtractor_name = "KNN"

    def on_timeout(self, *args):
        self.frame_processor.run()
        GObject.timeout_add(1, self.on_timeout, None)

    def main(self):
        counter = 0
        self.on_timeout()

        Gtk.main()

        if self.command:
            self.command.run()




import cv2
import time
import copy
import imutils
import numpy

from .contours import Contour


class Frame:
    def __init__(self, frame):
        self.frame = frame

    @property
    def width(self):
        return len(self.frame[0])

    @property
    def height(self):
        return len(self.frame)

    @property
    def area(self):
        return self.width * self.height

    def draw_top_label(self, text, color):
        cv2.putText(
            self.frame,
            text,
            (10, 25),
            cv2.FONT_HERSHEY_SIMPLEX,
            0.7,
            color,
            2
        )

    def draw_rect(self, point1, point2, color):
        cv2.rectangle(self.frame, point1, point2, color, 1)

    def draw_timer(self, time_struct):
        cv2.putText(
            self.frame,
            time.strftime("%H:%M:%S", time_struct),
            (10, self.height - 10),
            cv2.FONT_HERSHEY_SIMPLEX,
            0.35,
            (0, 0, 255),
            1
        )

    def resize(self, width):
        return Frame(imutils.resize(self.frame, width=width))

    def clone(self):
        return copy.deepcopy(self)

    def get_contours(self):
        contours = []
        (cnts, _) = cv2.findContours(self.frame, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)[-2:]
        for c in cnts:
            contours.append(Contour(c))
        return contours

    def blur(self, quantity):
        self.frame = cv2.blur(self.frame, (quantity, quantity))

    def threshold(self, minimun):
        self.frame = cv2.threshold(self.frame, minimun, 255, cv2.THRESH_BINARY)[1]

    def gray(self):
        self.frame = cv2.cvtColor(self.frame, cv2.COLOR_BGR2GRAY)

    def gray_copy(self):
        return Frame(cv2.cvtColor(self.frame, cv2.COLOR_BGR2GRAY))

    def crop(self, point1, point2):
        self.frame = self.frame[point1[1]:point2[1], point1[0]:point2[0]]

    def crop_copy(self, point1, point2):
        return Frame(self.frame[point1[1]:point2[1], point1[0]:point2[0]])

    def write(self, filename):
        cv2.imwrite(filename, self.frame)

    def gray_to_rgb(self):
        self.frame = cv2.cvtColor(self.frame, cv2.COLOR_GRAY2RGB)

    def bgr_to_rgb(self):
        self.frame = cv2.cvtColor(self.frame, cv2.COLOR_BGR2RGB)

    def gray_to_rgb_copy(self):
        return Frame(cv2.cvtColor(self.frame, cv2.COLOR_GRAY2RGB))

    def bgr_to_rgb_copy(self):
        return Frame(cv2.cvtColor(self.frame, cv2.COLOR_BGR2RGB))

    def to_string(self):
        return numpy.ndarray.tostring(self.frame)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name = "camtones",
    version = "0.0.1",
    description = "Camera To Not-Empty Sequence is a motion and face extration system",
    author = "Jesús Espino",
    author_email = "jespinog@gmail.com",
    url = "https://github.com/jespino/camtones",
    scripts = ['scripts/camtones'],
    packages = find_packages(),
    package_data={
        'camtones': [
            'haars/*',
            'procs/gui.glade',
        ]
    },
    install_requires=[
        'imutils',
        'numpy',
        'click',
    ],
    license = "BSD",
    classifiers = [
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.4',
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'License :: OSI Approved :: BSD License',
        'Operating System :: Unix',
    ],
)



import cv2


class Window:
    def __init__(self, name):
        self.name = name
        self.extra_handlers = {}
        cv2.namedWindow(name, cv2.WINDOW_NORMAL)

    def show(self, frame):
        cv2.imshow(self.name, frame.frame)

    def add_handler(self, key, handler):
        self.extra_handlers[key] = handler

    @property
    def is_fullscreen(self):
        return cv2.getWindowProperty(self.name, cv2.WND_PROP_FULLSCREEN) == cv2.WINDOW_FULLSCREEN

    def to_normal(self):
        cv2.setWindowProperty(self.name, cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_NORMAL)

    def to_fullscreen(self):
        cv2.setWindowProperty(self.name, cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)

    def handle(self, camera):
        key = cv2.waitKey(1) & 0xFF

        if key in self.extra_handlers:
            return self.extra_handlers[key](camera)

        if key == ord("q"):
            if camera.frames != -1:
                camera.seek(camera.frames)
            else:
                return False
        elif key == ord("f"):
            if self.is_fullscreen:
                self.to_normal()
            else:
                self.to_fullscreen()
        elif key == 81:  # left
            camera.seek(max(camera.current_frame - 100, 0))
        elif key == 82:  # up
            camera.seek(min(camera.current_frame + 1000, camera.frames))
        elif key == 83:  # right
            camera.seek(min(camera.current_frame + 100, camera.frames))
        elif key == 84:  # down
            camera.seek(max(camera.current_frame - 1000, 0))
        return True

    def __del__(self):
        cv2.destroyWindow(self.name)
